/*++

Copyright (c) 1999 - 2002  Microsoft Corporation

Module Name:

    passThrough.c

Abstract:

    This is the main module of the passThrough miniFilter driver.
    This filter hooks all IO operations for both pre and post operation
    callbacks.  The filter passes through the operations.

Environment:

    Kernel mode

--*/

#include <fltKernel.h>
#include <dontuse.h>
#include <suppress.h>
#include "base.h"

#pragma prefast(disable:__WARNING_ENCODE_MEMBER_FUNCTION_POINTER, "Not valid for kernel mode drivers")


PFLT_FILTER gFilterHandle;
PFLT_PORT 	gServerPort;
PFLT_PORT 	gClientPort;
ULONG_PTR OperationStatusCtx = 1;

#define PTDBG_TRACE_ROUTINES            0x00000001
#define PTDBG_TRACE_OPERATION_STATUS    0x00000002

ULONG gTraceFlags = 0;


#define PT_DBG_PRINT( _dbgLevel, _string )          \
    (FlagOn(gTraceFlags,(_dbgLevel)) ?              \
        DbgPrint _string :                          \
        ((int)0))
/************************************************************************/
/* ַ                                                           */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/************************************************************************/
#define FLPROF_PORT_NAME			L"\\FLProfPort"
#define FLIFE_REG_PATH				L"\\REGISTRY\\MACHINE\\SOFTWARE\\FTL\\FASTLIFE"
#define FLIFE_REG_NAME				L"AppPath"

/**
* UNICODE_STRINGתΪдCHAR[]
*
*/
BOOLEAN FLUnicodeStringToChar(PUNICODE_STRING UniName, PCHAR Name);

/**
* ȡעREG_SZֵ
*/
NTSTATUS RegReadStrValue(
						 __in_opt HANDLE KeyHandle,
						 __in PUNICODE_STRING ValueName,
						 __out PWCHAR Value
						 );
/**
* жչ
*/
BOOLEAN IsProtectExtName(
						 __in_opt PUNICODE_STRING UnicodeString
						 );

//  Defines the commands between the utility and the filter
typedef enum _NPMINI_COMMAND {
	ENUM_PASS = 0,
	ENUM_BLOCK
} NPMINI_COMMAND;	

//  Defines the command structure between the utility and the filter.
typedef struct _COMMAND_MESSAGE {
	NPMINI_COMMAND 	Command;  
} COMMAND_MESSAGE, *PCOMMAND_MESSAGE;

NTSTATUS
FLMiniMessage (
			   __in PVOID ConnectionCookie,
			   __in_bcount_opt(InputBufferSize) PVOID InputBuffer,
			   __in ULONG InputBufferSize,
			   __out_bcount_part_opt(OutputBufferSize,*ReturnOutputBufferLength) PVOID OutputBuffer,
			   __in ULONG OutputBufferSize,
			   __out PULONG ReturnOutputBufferLength
			   );

NTSTATUS
FLMiniConnect(
			  __in PFLT_PORT ClientPort,
			  __in PVOID ServerPortCookie,
			  __in_bcount(SizeOfContext) PVOID ConnectionContext,
			  __in ULONG SizeOfContext,
			  __deref_out_opt PVOID *ConnectionCookie
			  );

VOID
FLMiniDisconnect(
				 __in_opt PVOID ConnectionCookie
				 );


NPMINI_COMMAND gCommand = ENUM_PASS;
WCHAR FLPath[260]; //װ·
UNICODE_STRING FLinstallPath;

/*************************************************************************
    Prototypes
*************************************************************************/

DRIVER_INITIALIZE DriverEntry;
NTSTATUS
DriverEntry (
    __in PDRIVER_OBJECT DriverObject,
    __in PUNICODE_STRING RegistryPath
    );

NTSTATUS
PtInstanceSetup (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_SETUP_FLAGS Flags,
    __in DEVICE_TYPE VolumeDeviceType,
    __in FLT_FILESYSTEM_TYPE VolumeFilesystemType
    );

VOID
PtInstanceTeardownStart (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_TEARDOWN_FLAGS Flags
    );

VOID
PtInstanceTeardownComplete (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_TEARDOWN_FLAGS Flags
    );

NTSTATUS
PtUnload (
    __in FLT_FILTER_UNLOAD_FLAGS Flags
    );

NTSTATUS
PtInstanceQueryTeardown (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_QUERY_TEARDOWN_FLAGS Flags
    );

FLT_PREOP_CALLBACK_STATUS
PtPreOperationPassThrough (
    __inout PFLT_CALLBACK_DATA Data,
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __deref_out_opt PVOID *CompletionContext
    );

FLT_PREOP_CALLBACK_STATUS
FLPreDelete (
			 __inout PFLT_CALLBACK_DATA Data,
			 __in PCFLT_RELATED_OBJECTS FltObjects,
			 __deref_out_opt PVOID *CompletionContext
			 );

FLT_PREOP_CALLBACK_STATUS
FLPreWrite (
			__inout PFLT_CALLBACK_DATA Data,
			__in PCFLT_RELATED_OBJECTS FltObjects,
			__deref_out_opt PVOID *CompletionContext
			);

VOID
PtOperationStatusCallback (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in PFLT_IO_PARAMETER_BLOCK ParameterSnapshot,
    __in NTSTATUS OperationStatus,
    __in PVOID RequesterContext
    );

FLT_POSTOP_CALLBACK_STATUS
PtPostOperationPassThrough (
    __inout PFLT_CALLBACK_DATA Data,
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in_opt PVOID CompletionContext,
    __in FLT_POST_OPERATION_FLAGS Flags
    );

FLT_PREOP_CALLBACK_STATUS
PtPreOperationNoPostOperationPassThrough (
    __inout PFLT_CALLBACK_DATA Data,
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __deref_out_opt PVOID *CompletionContext
    );

BOOLEAN
PtDoRequestOperationStatus(
    __in PFLT_CALLBACK_DATA Data
    );

//
//  Assign text sections for each routine.
//

#ifdef ALLOC_PRAGMA
#pragma alloc_text(INIT, DriverEntry)
#pragma alloc_text(PAGE, PtUnload)
#pragma alloc_text(PAGE, PtInstanceQueryTeardown)
#pragma alloc_text(PAGE, PtInstanceSetup)
#pragma alloc_text(PAGE, PtInstanceTeardownStart)
#pragma alloc_text(PAGE, PtInstanceTeardownComplete)
#endif

//
//  operation registration
//

CONST FLT_OPERATION_REGISTRATION Callbacks[] = {
    { IRP_MJ_CREATE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_CREATE_NAMED_PIPE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_CLOSE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_READ,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_WRITE,
      FLTFL_OPERATION_REGISTRATION_SKIP_PAGING_IO,
      FLPreWrite,							//дļ
      PtPostOperationPassThrough },

    { IRP_MJ_QUERY_INFORMATION,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_SET_INFORMATION,
      0,
      FLPreDelete,							//ɾļļ
      PtPostOperationPassThrough },

    { IRP_MJ_QUERY_EA,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_SET_EA,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_FLUSH_BUFFERS,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_QUERY_VOLUME_INFORMATION,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_SET_VOLUME_INFORMATION,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_DIRECTORY_CONTROL,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_FILE_SYSTEM_CONTROL,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_DEVICE_CONTROL,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_INTERNAL_DEVICE_CONTROL,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_SHUTDOWN,
      0,
      PtPreOperationNoPostOperationPassThrough,
      NULL },                               //post operations not supported

    { IRP_MJ_LOCK_CONTROL,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_CLEANUP,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_CREATE_MAILSLOT,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_QUERY_SECURITY,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_SET_SECURITY,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_QUERY_QUOTA,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_SET_QUOTA,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_PNP,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_ACQUIRE_FOR_SECTION_SYNCHRONIZATION,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_RELEASE_FOR_SECTION_SYNCHRONIZATION,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_ACQUIRE_FOR_MOD_WRITE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_RELEASE_FOR_MOD_WRITE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_ACQUIRE_FOR_CC_FLUSH,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_RELEASE_FOR_CC_FLUSH,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_FAST_IO_CHECK_IF_POSSIBLE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_NETWORK_QUERY_OPEN,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_MDL_READ,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_MDL_READ_COMPLETE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_PREPARE_MDL_WRITE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_MDL_WRITE_COMPLETE,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_VOLUME_MOUNT,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_VOLUME_DISMOUNT,
      0,
      PtPreOperationPassThrough,
      PtPostOperationPassThrough },

    { IRP_MJ_OPERATION_END }
};

//
//  This defines what we want to filter with FltMgr
//

CONST FLT_REGISTRATION FilterRegistration = {

    sizeof( FLT_REGISTRATION ),         //  Size
    FLT_REGISTRATION_VERSION,           //  Version
    0,                                  //  Flags

    NULL,                               //  Context
    Callbacks,                          //  Operation callbacks

    PtUnload,                           //  MiniFilterUnload

    PtInstanceSetup,                    //  InstanceSetup
    PtInstanceQueryTeardown,            //  InstanceQueryTeardown
    PtInstanceTeardownStart,            //  InstanceTeardownStart
    PtInstanceTeardownComplete,         //  InstanceTeardownComplete

    NULL,                               //  GenerateFileName
    NULL,                               //  GenerateDestinationFileName
    NULL                                //  NormalizeNameComponent

};



NTSTATUS
PtInstanceSetup (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_SETUP_FLAGS Flags,
    __in DEVICE_TYPE VolumeDeviceType,
    __in FLT_FILESYSTEM_TYPE VolumeFilesystemType
    )
/*++

Routine Description:

    This routine is called whenever a new instance is created on a volume. This
    gives us a chance to decide if we need to attach to this volume or not.

    If this routine is not defined in the registration structure, automatic
    instances are alwasys created.

Arguments:

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance and its associated volume.

    Flags - Flags describing the reason for this attach request.

Return Value:

    STATUS_SUCCESS - attach
    STATUS_FLT_DO_NOT_ATTACH - do not attach

--*/
{
    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( Flags );
    UNREFERENCED_PARAMETER( VolumeDeviceType );
    UNREFERENCED_PARAMETER( VolumeFilesystemType );

    PAGED_CODE();

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtInstanceSetup: Entered\n") );

    return STATUS_SUCCESS;
}


NTSTATUS
PtInstanceQueryTeardown (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_QUERY_TEARDOWN_FLAGS Flags
    )
/*++

Routine Description:

    This is called when an instance is being manually deleted by a
    call to FltDetachVolume or FilterDetach thereby giving us a
    chance to fail that detach request.

    If this routine is not defined in the registration structure, explicit
    detach requests via FltDetachVolume or FilterDetach will always be
    failed.

Arguments:

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance and its associated volume.

    Flags - Indicating where this detach request came from.

Return Value:

    Returns the status of this operation.

--*/
{
    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( Flags );

    PAGED_CODE();

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtInstanceQueryTeardown: Entered\n") );

    return STATUS_SUCCESS;
}


VOID
PtInstanceTeardownStart (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_TEARDOWN_FLAGS Flags
    )
/*++

Routine Description:

    This routine is called at the start of instance teardown.

Arguments:

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance and its associated volume.

    Flags - Reason why this instance is been deleted.

Return Value:

    None.

--*/
{
    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( Flags );

    PAGED_CODE();

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtInstanceTeardownStart: Entered\n") );
}


VOID
PtInstanceTeardownComplete (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_TEARDOWN_FLAGS Flags
    )
/*++

Routine Description:

    This routine is called at the end of instance teardown.

Arguments:

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance and its associated volume.

    Flags - Reason why this instance is been deleted.

Return Value:

    None.

--*/
{
    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( Flags );

    PAGED_CODE();

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtInstanceTeardownComplete: Entered\n") );
}


/*************************************************************************
    MiniFilter initialization and unload routines.
*************************************************************************/

NTSTATUS
DriverEntry (
    __in PDRIVER_OBJECT DriverObject,
    __in PUNICODE_STRING RegistryPath
    )
/*++

Routine Description:

    This is the initialization routine for this miniFilter driver.  This
    registers with FltMgr and initializes all global data structures.

Arguments:

    DriverObject - Pointer to driver object created by the system to
        represent this driver.

    RegistryPath - Unicode string identifying where the parameters for this
        driver are located in the registry.

Return Value:

    Returns STATUS_SUCCESS.

--*/
{
    NTSTATUS status;

	PSECURITY_DESCRIPTOR sd;
	OBJECT_ATTRIBUTES oa;
	UNICODE_STRING uniString;		//ͨ
	UNICODE_STRING KeyName;
	UNICODE_STRING ValueName;
	HANDLE KeyHandle;

	UNICODE_STRING UFLPath;


    UNREFERENCED_PARAMETER( RegistryPath );

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!DriverEntry: Entered\n") );

	//עȡFLifeװ·
	RtlInitUnicodeString(&KeyName,FLIFE_REG_PATH);
	RtlInitUnicodeString(&ValueName,FLIFE_REG_NAME);

	InitializeObjectAttributes(
		&oa,
		&KeyName,
		OBJ_CASE_INSENSITIVE | OBJ_KERNEL_HANDLE,
		NULL,
		NULL
		);
	status = ZwOpenKey(
		&KeyHandle,
		KEY_READ,
		&oa
		);

	if(!NT_SUCCESS(status))
	{
		DbgPrint("Unable to open key %08x\n",status);
		KeyHandle = NULL;
	}

	if( !NT_SUCCESS(RegReadStrValue(KeyHandle,&ValueName,FLPath)))
	{
		return status;
	}
	
	
	RtlInitUnicodeString(&FLinstallPath,FLPath + 6);

	if( KeyHandle)
		ZwClose(KeyHandle);
    //
    //  Register with FltMgr to tell it our callback routines
    //

    status = FltRegisterFilter( DriverObject,
                                &FilterRegistration,
                                &gFilterHandle );

    ASSERT( NT_SUCCESS( status ) );

    if (NT_SUCCESS( status )) {

        //
        //  Start filtering i/o
        //

        status = FltStartFiltering( gFilterHandle );

        if (!NT_SUCCESS( status )) {

            FltUnregisterFilter( gFilterHandle );
        }
    }

	//Communication Port
	status  = FltBuildDefaultSecurityDescriptor( &sd, FLT_PORT_ALL_ACCESS );

	if (!NT_SUCCESS( status )) {
		goto final;
	}


	status  = FltBuildDefaultSecurityDescriptor( &sd, FLT_PORT_ALL_ACCESS );

	if (!NT_SUCCESS( status )) {
		goto final;
	}


	RtlInitUnicodeString( &uniString, FLPROF_PORT_NAME );

	InitializeObjectAttributes( &oa,
		&uniString,
		OBJ_KERNEL_HANDLE | OBJ_CASE_INSENSITIVE,
		NULL,
		sd );

	status = FltCreateCommunicationPort( gFilterHandle,
		&gServerPort,
		&oa,
		NULL,
		FLMiniConnect,
		FLMiniDisconnect,
		FLMiniMessage,
		1 );

	FltFreeSecurityDescriptor( sd );

	if (!NT_SUCCESS( status )) {
		goto final;
	}            

final :

	if (!NT_SUCCESS( status ) ) {

		if (NULL != gServerPort) {
			FltCloseCommunicationPort( gServerPort );
		}

		if (NULL != gFilterHandle) {
			FltUnregisterFilter( gFilterHandle );
		}       
	} 

    return status;
}

NTSTATUS
PtUnload (
    __in FLT_FILTER_UNLOAD_FLAGS Flags
    )
/*++

Routine Description:

    This is the unload routine for this miniFilter driver. This is called
    when the minifilter is about to be unloaded. We can fail this unload
    request if this is not a mandatory unloaded indicated by the Flags
    parameter.

Arguments:

    Flags - Indicating if this is a mandatory unload.

Return Value:

    Returns the final status of this operation.

--*/
{
    UNREFERENCED_PARAMETER( Flags );

    PAGED_CODE();

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtUnload: Entered\n") );

	FltCloseCommunicationPort( gServerPort );

    FltUnregisterFilter( gFilterHandle );




    return STATUS_SUCCESS;
}


/*************************************************************************
    MiniFilter callback routines.
*************************************************************************/
FLT_PREOP_CALLBACK_STATUS
PtPreOperationPassThrough (
    __inout PFLT_CALLBACK_DATA Data,
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __deref_out_opt PVOID *CompletionContext
    )
/*++

Routine Description:

    This routine is the main pre-operation dispatch routine for this
    miniFilter. Since this is just a simple passThrough miniFilter it
    does not do anything with the callbackData but rather return
    FLT_PREOP_SUCCESS_WITH_CALLBACK thereby passing it down to the next
    miniFilter in the chain.

    This is non-pageable because it could be called on the paging path

Arguments:

    Data - Pointer to the filter callbackData that is passed to us.

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance, its associated volume and
        file object.

    CompletionContext - The context for the completion routine for this
        operation.

Return Value:

    The return value is the status of the operation.

--*/
{
    NTSTATUS status;

    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( CompletionContext );

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtPreOperationPassThrough: Entered\n") );

    //
    //  See if this is an operation we would like the operation status
    //  for.  If so request it.
    //
    //  NOTE: most filters do NOT need to do this.  You only need to make
    //        this call if, for example, you need to know if the oplock was
    //        actually granted.
    //

    if (PtDoRequestOperationStatus( Data )) {

        status = FltRequestOperationStatusCallback( Data,
                                                    PtOperationStatusCallback,
                                                    (PVOID)(++OperationStatusCtx) );
        if (!NT_SUCCESS(status)) {

            PT_DBG_PRINT( PTDBG_TRACE_OPERATION_STATUS,
                          ("PassThrough!PtPreOperationPassThrough: FltRequestOperationStatusCallback Failed, status=%08x\n",
                           status) );
        }
    }

    return FLT_PREOP_SUCCESS_WITH_CALLBACK;
}

__checkReturn
BOOLEAN
IsSkipPreWrite (
				__in PFLT_CALLBACK_DATA Data,
				__in PCFLT_RELATED_OBJECTS FltObjects
				)
{
	if ( !FltObjects->Instance )
	{
		return TRUE;
	}

	if ( !FltObjects->FileObject )
	{
		return TRUE;
	}

	if ( FlagOn( FltObjects->FileObject->Flags, FO_NAMED_PIPE ) )
	{
		return TRUE;
	}

	return FALSE;
}


FLT_PREOP_CALLBACK_STATUS
FLPreDelete (
			 __inout PFLT_CALLBACK_DATA Data,
			 __in PCFLT_RELATED_OBJECTS FltObjects,
			 __deref_out_opt PVOID *CompletionContext
			 )
{
//	char FileName[260] = "X:";
//	char FLPath[260] = "C:";

	NTSTATUS status;
	PFLT_FILE_NAME_INFORMATION nameInfo;

	UNREFERENCED_PARAMETER( FltObjects );
	UNREFERENCED_PARAMETER( CompletionContext );

	PAGED_CODE();      

	if (FltObjects->FileObject ==NULL)
		return FLT_PREOP_SUCCESS_NO_CALLBACK;

	if (!Data)
		return FLT_PREOP_SUCCESS_NO_CALLBACK;

	//ɾļ
	if( Data->Iopb->Parameters.SetFileInformation.FileInformationClass == FileDispositionInformation || Data->Iopb->Parameters.SetFileInformation.FileInformationClass == FileRenameInformation)
	{



		__try {	                             								
			status = FltGetFileNameInformation( Data,
				FLT_FILE_NAME_NORMALIZED |
				FLT_FILE_NAME_QUERY_DEFAULT,
				&nameInfo );
			if (NT_SUCCESS( status )) {
				//P_O_
				if (gCommand == ENUM_BLOCK)
				{
					if( nameInfo->NamesParsed == 4)
						FltParseFileNameInformation( nameInfo );

					//_asm int 3;
					//ڰװĿ¼
				//	if (wcsstr(nameInfo->Name.Buffer, FLPath+6) > 0)
					if( PhFindStringInStringRef(&nameInfo->Name, &FLinstallPath, TRUE) != -1)
					{
						DbgPrint("in install dir\n");
						if(IsProtectExtName(&nameInfo->Name))
						{
							DbgPrint("ext ext txt \n");
							Data->IoStatus.Status = STATUS_ACCESS_DENIED;
							Data->IoStatus.Information = 0;
							FltReleaseFileNameInformation( nameInfo );
							return FLT_PREOP_COMPLETE;	
						}

					}						 
				}   			    
				//release resource
				FltReleaseFileNameInformation( nameInfo );        
			}                                    
		}
		__except(EXCEPTION_EXECUTE_HANDLER) {
			DbgPrint("NPPreCreate EXCEPTION_EXECUTE_HANDLER\n");				
		}

	}

	return FLT_PREOP_SUCCESS_WITH_CALLBACK;
}

FLT_PREOP_CALLBACK_STATUS
FLPreWrite (
			__inout PFLT_CALLBACK_DATA Data,
			__in PCFLT_RELATED_OBJECTS FltObjects,
			__deref_out_opt PVOID *CompletionContext
			)
{

	NTSTATUS status;
	PFLT_FILE_NAME_INFORMATION nameInfo;

	UNREFERENCED_PARAMETER( FltObjects );
	UNREFERENCED_PARAMETER( CompletionContext );

	PAGED_CODE();      


	if ( IsSkipPreWrite( Data, FltObjects ) )
	{
		return FLT_PREOP_SUCCESS_NO_CALLBACK;
	}

	__try {	                             								
		status = FltGetFileNameInformation( Data,
			FLT_FILE_NAME_NORMALIZED |
			FLT_FILE_NAME_QUERY_DEFAULT,
			&nameInfo );
		if (NT_SUCCESS( status )) 
		{
			//P_O_
			if (gCommand == ENUM_BLOCK) 
			{
				status = FltParseFileNameInformation( nameInfo );
				if(!NT_SUCCESS(status))
						return status;
				//ڰװĿ¼
				
				if( PhFindStringInStringRef(&nameInfo->Name, &FLinstallPath, TRUE) != -1)
				{
					DbgPrint("in install dir\n");
					if(IsProtectExtName(&nameInfo->Name))
					{
						DbgPrint("ext ext txt \n");
						Data->IoStatus.Status = STATUS_ACCESS_DENIED;
						Data->IoStatus.Information = 0;
						FltReleaseFileNameInformation( nameInfo );
						return FLT_PREOP_COMPLETE;	
					}

				}	
				
			}   			    
			//release resource
			FltReleaseFileNameInformation( nameInfo );        
		}                                    
	}
	__except(EXCEPTION_EXECUTE_HANDLER) {
		DbgPrint("NPPreWrite EXCEPTION_EXECUTE_HANDLER\n");				
	}


	return FLT_PREOP_SUCCESS_WITH_CALLBACK;
}



VOID
PtOperationStatusCallback (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in PFLT_IO_PARAMETER_BLOCK ParameterSnapshot,
    __in NTSTATUS OperationStatus,
    __in PVOID RequesterContext
    )
/*++

Routine Description:

    This routine is called when the given operation returns from the call
    to IoCallDriver.  This is useful for operations where STATUS_PENDING
    means the operation was successfully queued.  This is useful for OpLocks
    and directory change notification operations.

    This callback is called in the context of the originating thread and will
    never be called at DPC level.  The file object has been correctly
    referenced so that you can access it.  It will be automatically
    dereferenced upon return.

    This is non-pageable because it could be called on the paging path

Arguments:

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance, its associated volume and
        file object.

    RequesterContext - The context for the completion routine for this
        operation.

    OperationStatus -

Return Value:

    The return value is the status of the operation.

--*/
{
    UNREFERENCED_PARAMETER( FltObjects );

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtOperationStatusCallback: Entered\n") );

    PT_DBG_PRINT( PTDBG_TRACE_OPERATION_STATUS,
                  ("PassThrough!PtOperationStatusCallback: Status=%08x ctx=%p IrpMj=%02x.%02x \"%s\"\n",
                   OperationStatus,
                   RequesterContext,
                   ParameterSnapshot->MajorFunction,
                   ParameterSnapshot->MinorFunction,
                   FltGetIrpName(ParameterSnapshot->MajorFunction)) );
}


FLT_POSTOP_CALLBACK_STATUS
PtPostOperationPassThrough (
    __inout PFLT_CALLBACK_DATA Data,
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in_opt PVOID CompletionContext,
    __in FLT_POST_OPERATION_FLAGS Flags
    )
/*++

Routine Description:

    This routine is the post-operation completion routine for this
    miniFilter.

    This is non-pageable because it may be called at DPC level.

Arguments:

    Data - Pointer to the filter callbackData that is passed to us.

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance, its associated volume and
        file object.

    CompletionContext - The completion context set in the pre-operation routine.

    Flags - Denotes whether the completion is successful or is being drained.

Return Value:

    The return value is the status of the operation.

--*/
{
    UNREFERENCED_PARAMETER( Data );
    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( CompletionContext );
    UNREFERENCED_PARAMETER( Flags );

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtPostOperationPassThrough: Entered\n") );

    return FLT_POSTOP_FINISHED_PROCESSING;
}


FLT_PREOP_CALLBACK_STATUS
PtPreOperationNoPostOperationPassThrough (
    __inout PFLT_CALLBACK_DATA Data,
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __deref_out_opt PVOID *CompletionContext
    )
/*++

Routine Description:

    This routine is the main pre-operation dispatch routine for this
    miniFilter. Since this is just a simple passThrough miniFilter it
    does not do anything with the callbackData but rather return
    FLT_PREOP_SUCCESS_WITH_CALLBACK thereby passing it down to the next
    miniFilter in the chain.

    This is non-pageable because it could be called on the paging path

Arguments:

    Data - Pointer to the filter callbackData that is passed to us.

    FltObjects - Pointer to the FLT_RELATED_OBJECTS data structure containing
        opaque handles to this filter, instance, its associated volume and
        file object.

    CompletionContext - The context for the completion routine for this
        operation.

Return Value:

    The return value is the status of the operation.

--*/
{
    UNREFERENCED_PARAMETER( Data );
    UNREFERENCED_PARAMETER( FltObjects );
    UNREFERENCED_PARAMETER( CompletionContext );

    PT_DBG_PRINT( PTDBG_TRACE_ROUTINES,
                  ("PassThrough!PtPreOperationNoPostOperationPassThrough: Entered\n") );

    return FLT_PREOP_SUCCESS_NO_CALLBACK;
}


BOOLEAN
PtDoRequestOperationStatus(
    __in PFLT_CALLBACK_DATA Data
    )
/*++

Routine Description:

    This identifies those operations we want the operation status for.  These
    are typically operations that return STATUS_PENDING as a normal completion
    status.

Arguments:

Return Value:

    TRUE - If we want the operation status
    FALSE - If we don't

--*/
{
    PFLT_IO_PARAMETER_BLOCK iopb = Data->Iopb;

    //
    //  return boolean state based on which operations we are interested in
    //

    return (BOOLEAN)

            //
            //  Check for oplock operations
            //

             (((iopb->MajorFunction == IRP_MJ_FILE_SYSTEM_CONTROL) &&
               ((iopb->Parameters.FileSystemControl.Common.FsControlCode == FSCTL_REQUEST_FILTER_OPLOCK)  ||
                (iopb->Parameters.FileSystemControl.Common.FsControlCode == FSCTL_REQUEST_BATCH_OPLOCK)   ||
                (iopb->Parameters.FileSystemControl.Common.FsControlCode == FSCTL_REQUEST_OPLOCK_LEVEL_1) ||
                (iopb->Parameters.FileSystemControl.Common.FsControlCode == FSCTL_REQUEST_OPLOCK_LEVEL_2)))

              ||

              //
              //    Check for directy change notification
              //

              ((iopb->MajorFunction == IRP_MJ_DIRECTORY_CONTROL) &&
               (iopb->MinorFunction == IRP_MN_NOTIFY_CHANGE_DIRECTORY))
             );
}

//Puser application Conect
NTSTATUS
FLMiniConnect(
			  __in PFLT_PORT ClientPort,
			  __in PVOID ServerPortCookie,
			  __in_bcount(SizeOfContext) PVOID ConnectionContext,
			  __in ULONG SizeOfContext,
			  __deref_out_opt PVOID *ConnectionCookie
			  )
{
	DbgPrint("[mini-filter] NPMiniConnect");
	PAGED_CODE();

	UNREFERENCED_PARAMETER( ServerPortCookie );
	UNREFERENCED_PARAMETER( ConnectionContext );
	UNREFERENCED_PARAMETER( SizeOfContext);
	UNREFERENCED_PARAMETER( ConnectionCookie );

	ASSERT( gClientPort == NULL );
	gClientPort = ClientPort;
	return STATUS_SUCCESS;
}

//Puser application Disconect
VOID
FLMiniDisconnect(
				 __in_opt PVOID ConnectionCookie
				 )
{		
	PAGED_CODE();
	UNREFERENCED_PARAMETER( ConnectionCookie );
	DbgPrint("[mini-filter] NPMiniDisconnect");

	//  Close our handle
	FltCloseClientPort( gFilterHandle, &gClientPort );
}

//log message with filter mask "IHVDRIVER" and severity level "INFO"
void DebugInfo(char *str) {
	DbgPrintEx(DPFLTR_IHVDRIVER_ID, DPFLTR_INFO_LEVEL, "testdrv: %s\n", str);
}

NTSTATUS
FLMiniMessage (
			   __in PVOID ConnectionCookie,
			   __in_bcount_opt(InputBufferSize) PVOID InputBuffer,
			   __in ULONG InputBufferSize,
			   __out_bcount_part_opt(OutputBufferSize,*ReturnOutputBufferLength) PVOID OutputBuffer,
			   __in ULONG OutputBufferSize,
			   __out PULONG ReturnOutputBufferLength
			   )
{

	NPMINI_COMMAND command;
	NTSTATUS status;

	PAGED_CODE();

	UNREFERENCED_PARAMETER( ConnectionCookie );
	UNREFERENCED_PARAMETER( OutputBufferSize );
	UNREFERENCED_PARAMETER( OutputBuffer );

	DbgPrint("[mini-filter] NPMiniMessage");

	//                      **** PLEASE READ ****    
	//  The INPUT and OUTPUT buffers are raw user mode addresses.  The filter
	//  manager has already done a ProbedForRead (on InputBuffer) and
	//  ProbedForWrite (on OutputBuffer) which guarentees they are valid
	//  addresses based on the access (user mode vs. kernel mode).  The
	//  minifilter does not need to do their own probe.
	//  The filter manager is NOT doing any alignment checking on the pointers.
	//  The minifilter must do this themselves if they care (see below).
	//  The minifilter MUST continue to use a try/except around any access to
	//  these buffers.    

#if defined(_WIN64)
#endif

	if ((InputBuffer != NULL) &&
		(InputBufferSize >= (FIELD_OFFSET(COMMAND_MESSAGE,Command) +
		sizeof(NPMINI_COMMAND)))) {

			try  {
				//  Probe and capture input message: the message is raw user mode
				//  buffer, so need to protect with exception handler
				command = ((PCOMMAND_MESSAGE) InputBuffer)->Command;

			} except( EXCEPTION_EXECUTE_HANDLER ) {

				return GetExceptionCode();
			}

			switch (command) {
				//}Wh
			case ENUM_PASS:
				{            		
					DebugInfo("[mini-filter] ENUM_PASS");
					gCommand = ENUM_PASS;
					status = STATUS_SUCCESS;         
					break;
				}
				//׳Wh
			case ENUM_BLOCK:
				{            		
					DebugInfo("[mini-filter] ENUM_BLOCK");
					gCommand = ENUM_BLOCK;
					status = STATUS_SUCCESS;                          
					break;
				}               		

			default:
				
				DebugInfo("[mini-filter] default");
				status = STATUS_INVALID_PARAMETER;
				break;
			}
	} else {

		status = STATUS_INVALID_PARAMETER;
	}

	return status;
}


BOOLEAN FLUnicodeStringToChar(PUNICODE_STRING UniName, PCHAR Name)
{
	ANSI_STRING	AnsiName;			
	NTSTATUS	ntstatus;
	char*		nameptr;			

	__try {	    		   		    		
		ntstatus = RtlUnicodeStringToAnsiString(&AnsiName, UniName, TRUE);		

		if (AnsiName.Length < 260) {
			nameptr = (PCHAR)AnsiName.Buffer;
			//Convert into upper case and copy to buffer
			strcpy(Name, _strupr(nameptr));						    	    
			DbgPrint("NPUnicodeStringToChar : %s\n", Name);	
		}		  	
		RtlFreeAnsiString(&AnsiName);		 
	} 
	__except(EXCEPTION_EXECUTE_HANDLER) {
		DbgPrint("NPUnicodeStringToChar EXCEPTION_EXECUTE_HANDLER\n");	
		return FALSE;
	}
	return TRUE;
}  

NTSTATUS RegReadStrValue(
						 __in_opt HANDLE KeyHandle,
						 __in PUNICODE_STRING ValueName,
						 __out PWCHAR Value
						 )
{
	NTSTATUS status;
	WCHAR buffer[256];
	PKEY_VALUE_PARTIAL_INFORMATION info;
	ULONG resultLength;

	PAGED_CODE();

	if(!KeyHandle)
		return STATUS_INVALID_HANDLE;

	info = (PKEY_VALUE_PARTIAL_INFORMATION)buffer;

	status = ZwQueryValueKey(
		KeyHandle,
		ValueName,
		KeyValuePartialInformation,
		info,
		sizeof(buffer),
		&resultLength
		);

	if(info->Type != REG_SZ)
		status = STATUS_OBJECT_TYPE_MISMATCH;

	if(!NT_SUCCESS(status))
	{
		DbgPrint("Unable to query parameter %08x\n",status);
		return status;
	}
	RtlCopyMemory(Value,info->Data,info->DataLength);

	return STATUS_SUCCESS;
}



BOOLEAN IsProtectExtName(
						 __in_opt PUNICODE_STRING UnicodeString
						 )
{
//	PPH_STRING FileName;
//	FileName = PhCreateStringEx(UnicodeString->Buffer,UnicodeString->Length);

	if( PhEndsWithStringRef2(UnicodeString, L".dll",TRUE))
		return 1;
	else if( PhEndsWithStringRef2(UnicodeString, L".exe",TRUE))
		return 1;
	else if( PhEndsWithStringRef2(UnicodeString, L".sys",TRUE))
		return 1;
	return 0;
}


#ifdef _M_IX86

__declspec(naked) int __cdecl ph_equal_string(wchar_t *s1, wchar_t *s2, size_t len)
{
	__asm
	{
		push    esi
			push    edi

			mov     ecx, [esp+0xc+0x8] // len
		mov     eax, [esp+0xc+0x8] // len

		and     ecx, -2 // round down to even number
			mov     esi, [esp+0xc+0x0] // s1
		mov     edi, [esp+0xc+0x4] // s2
		jz      zero_or_one

loop_start:
		mov     edx, [esi]
		cmp     edx, [edi]
		jnz     return0

			add     esi, 4
			add     edi, 4
			sub     ecx, 2
			jnz     loop_start

			// Either we've reached the end, or the length was odd.
			test    eax, 1
			jnz     odd_length

return1:
		mov     eax, 1
			pop     edi
			pop     esi
			ret

zero_or_one:
		test    eax, eax
			jz      return1

odd_length:
		mov     dx, [esi]
		cmp     dx, [edi]
		jz      return1

return0:
		xor     eax, eax
			pop     edi
			pop     esi
			ret
	}
}
/*
__declspec(naked) unsigned long __cdecl ph_crc32(unsigned long crc, char *buf, size_t len)
{
	__asm
	{
		push    esi

			mov     eax, [esp+0x8+0x0] // crc
		mov     ecx, [esp+0x8+0x8] // len
		mov     esi, [esp+0x8+0x4] // buf

		xor     edx, edx
			jecxz   done
			not     eax

			// while (len--)
			//     crc = (crc >> 8) ^ table[(crc ^ *buf++) & 0xff];
loop_start:
		mov     dl, [esi]
		inc     esi
			xor     dl, al
			shr     eax, 8
			xor     eax, [PhCrc32Table+edx*4]

		sub     ecx, 1
			jnz     loop_start

			not     eax
done:
		pop     esi
			ret
	}
}
*/
__declspec(naked) unsigned short __cdecl ph_chksum(unsigned long sum, unsigned short *buf, unsigned long count)
{
	__asm
	{
		push    esi

			mov     ecx, [esp+0x8+0x8] // count
		mov     eax, [esp+0x8+0x0] // sum
		mov     esi, [esp+0x8+0x4] // buf

		// The checksum involves summing the words in the buffer, adding the carry back
		// onto the low word of the checksum. We can do this more efficiently by
		// working with dwords.

		// Make sure the buffer has 4 byte alignment.
		test    esi, 0x2
			jz      do_2

			// Not aligned - process one word.
			add     ax, [esi]
		adc     ax, 0
			add     esi, 2
			sub     ecx, 1

			// Since the buffer is aligned, we start clearing bits of the
			// count starting with 2 words. Once we reach 16 words at a time,
			// the higher bits are cleared by a loop. The remaining 1 word is
			// then cleared.

do_2:
		// 2 words
		test    ecx, 2
			jz      do_4

			add     eax, [esi]
		adc     eax, 0

			add     esi, 4
			sub     ecx, 2

do_4:
		// 4 words
		test    ecx, 4
			jz      do_8

			add     eax, [esi]
		adc     eax, [esi+4]
		adc     eax, 0

			add     esi, 8
			sub     ecx, 4

do_8:
		// 8 words
		test    ecx, 8
			jz      do_16

			add     eax, [esi]
		adc     eax, [esi+4]
		adc     eax, [esi+8]
		adc     eax, [esi+12]
		adc     eax, 0

			add     esi, 16
			sub     ecx, 8

do_16:
		// 16 words
		mov     edx, ecx
			and     edx, 15
			sub     ecx, edx // ignore smaller chunks
			jz      do_1

do_16_loop_start:
		add     eax, [esi]
		adc     eax, [esi+4]
		adc     eax, [esi+8]
		adc     eax, [esi+12]
		adc     eax, [esi+16]
		adc     eax, [esi+20]
		adc     eax, [esi+24]
		adc     eax, [esi+28]
		adc     eax, 0

			add     esi, 32
			sub     ecx, 16
			jnz     do_16_loop_start

do_1:
		// 1 word
		test    edx, 1
			jz      done

			add     ax, [esi]
		adc     ax, 0

done:
		// Merge the top 16 bits with the bottom 16 bits.
		mov     dx, ax
			shr     eax, 16
			add     ax, dx
			adc     ax, 0

			pop     esi
			ret
	}
}

#else

// CRC32 (IEEE 802.3)

DECLSPEC_SELECTANY
ULONG PhCrc32Table[256] =
{
	0x00000000, 0x77073096, 0xee0e612c, 0x990951ba, 0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
	0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988, 0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
	0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de, 0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
	0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec, 0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
	0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172, 0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
	0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940, 0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
	0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116, 0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
	0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924, 0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
	0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a, 0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
	0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818, 0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
	0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e, 0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
	0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c, 0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
	0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2, 0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
	0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0, 0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
	0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086, 0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
	0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4, 0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
	0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a, 0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
	0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8, 0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
	0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe, 0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
	0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc, 0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
	0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252, 0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
	0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60, 0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
	0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236, 0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
	0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04, 0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
	0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a, 0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
	0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38, 0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
	0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e, 0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
	0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c, 0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
	0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2, 0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
	0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0, 0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
	0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6, 0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
	0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94, 0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
};

int __cdecl ph_equal_string(wchar_t *s1, wchar_t *s2, size_t len)
{
	size_t l;

	l = len & -2; // round down to power of 2

	if (l)
	{
		while (TRUE)
		{
			if (*(PULONG)s1 != *(PULONG)s2)
				return 0;

			s1 += 2;
			s2 += 2;
			l -= 2;

			if (!l)
				break;
		}
	}

	if (len & 1)
		return *s1 == *s2;
	else
		return 1;
}

unsigned long __cdecl ph_crc32(unsigned long crc, char *buf, size_t len)
{
	crc ^= 0xffffffff;

	while (len--)
		crc = (crc >> 8) ^ PhCrc32Table[(crc ^ *buf++) & 0xff];

	return crc ^ 0xffffffff;
}

unsigned short __cdecl ph_chksum(unsigned long sum, unsigned short *buf, unsigned long count)
{
	while (count--)
	{
		sum += *buf++;
		sum = (sum >> 16) + (sum & 0xffff);
	}

	sum = (sum >> 16) + sum;

	return (unsigned short)sum;
}

#endif


/**
* Locates a string in a string.
*
* \param String1 The string to search.
* \param String2 The string to search for.
* \param IgnoreCase TRUE to perform a case-insensitive search, otherwise
* FALSE.
*
* \return The index, in characters, of the first occurrence of
* \a String2 in \a String1 after \a StartIndex. If \a String2 was not
* found, -1 is returned.
*/
ULONG PhFindStringInStringRef(
							  __in PUNICODE_STRING String1,
							  __in PUNICODE_STRING String2,
							  __in BOOLEAN IgnoreCase
							  )
{
	PWSTR string1;
	ULONG length1;
	PWSTR string2;
	ULONG length2;
	ULONG i;

	string1 = String1->Buffer;
	length1 = String1->Length / sizeof(WCHAR);
	string2 = String2->Buffer;
	length2 = String2->Length / sizeof(WCHAR);

	// Can't be a substring if it's bigger than the first string.
	if (length2 > length1)
		return -1;

	if (!IgnoreCase)
	{
		for (i = length1 - length2 + 1; i != 0; i--)
		{
			PWSTR s1;
			PWSTR s2;
			ULONG l2;

			s1 = string1;
			s2 = string2;
			l2 = length2 & -2;

			if (l2)
			{
				do
				{
					if (*(PULONG)s1 != *(PULONG)s2)
						goto ContinueLoop;

					s1 += 2;
					s2 += 2;
					l2 -= 2;
				} while (l2);
			}

			if (length2 & 1)
			{
				if (*s1 == *s2)
					goto FoundString;
			}
			else
			{
				goto FoundString;
			}

ContinueLoop:
			string1++;
		}

		return -1;
FoundString:
		return (ULONG)(string1 - String1->Buffer);
	}
	else
	{
		UNICODE_STRING us1;
		UNICODE_STRING us2;

		RtlInitUnicodeString(&us1,String1->Buffer);
		RtlInitUnicodeString(&us2,String2->Buffer);
	//	us1 = String1;
	//	us2 = String2;
		us1.Length = us2.Length;

		for (i = length1 - length2 + 1; i != 0; i--)
		{
			if (RtlEqualUnicodeString(&us1, &us2, TRUE))
				goto FoundUString;

			us1.Buffer++;
		}

		return -1;
FoundUString:
		return (ULONG)(us1.Buffer - String1->Buffer);
	}
}