
struct _PH_STRING;
typedef struct _PH_STRING *PPH_STRING;

typedef struct _PH_STRINGREF
{
	union
	{
		struct
		{
			/** The length, in bytes, of the string. */
			USHORT Length;
			/** Unused and of an undefined value. */
			USHORT Reserved;
			/** The buffer containing the contents of the string. */
			PWSTR Buffer;
		};
		UNICODE_STRING us;
	};
} PH_STRINGREF, *PPH_STRINGREF;

// heap

VOID
NTAPI
PhFree(
	   __in __post_invalid PVOID Memory
	   );

PPH_STRING
NTAPI
PhCreateStringEx(
				 __in_opt PWSTR Buffer,
				 __in SIZE_T Length
				 );

// baseupx (basic)

PPH_STRING
NTAPI
PhaLowerString(
			   __in PPH_STRING String
			   );

int
__cdecl
ph_equal_string(wchar_t *s1, wchar_t *s2, size_t len);

FORCEINLINE VOID PhInitializeStringRef(
									   __out PPH_STRINGREF String,
									   __in PWSTR Buffer
									   )
{
	String->Length = (USHORT)(wcslen(Buffer) * sizeof(WCHAR));
	String->Buffer = Buffer;
}

FORCEINLINE BOOLEAN PhEqualStringRef(
									 __in PUNICODE_STRING String1,
									 __in PUNICODE_STRING String2,
									 __in BOOLEAN IgnoreCase
									 )
{
	if (!IgnoreCase)
	{
		USHORT length1;

		length1 = String1->Length;

		if (length1 != String2->Length)
			return FALSE;

		return ph_equal_string(String1->Buffer, String2->Buffer, length1 / sizeof(WCHAR));
	}
	else
	{
		return RtlEqualUnicodeString(String1, String2, TRUE);
	}
}

FORCEINLINE BOOLEAN PhEndsWithStringRef(
										__in PUNICODE_STRING String1,
										__in PUNICODE_STRING String2,
										__in BOOLEAN IgnoreCase
										)
{
	UNICODE_STRING sr1;

	if (String2->Length > String1->Length)
		return FALSE;

	sr1.Buffer = &String1->Buffer[(String1->Length - String2->Length) / sizeof(WCHAR)];
	sr1.Length = String2->Length;

	return PhEqualStringRef(&sr1, String2, IgnoreCase);
}

FORCEINLINE BOOLEAN PhEndsWithStringRef2(
	__in PUNICODE_STRING String1,
	__in PWSTR String2,
	__in BOOLEAN IgnoreCase
	)
{
	UNICODE_STRING sr2;

	RtlInitUnicodeString(&sr2, String2);

	return PhEndsWithStringRef(String1, &sr2, IgnoreCase);
}

ULONG PhFindStringInStringRef(
							  __in PUNICODE_STRING String1,
							  __in PUNICODE_STRING String2,
							  __in BOOLEAN IgnoreCase
							  );