#include "MemSearch.h"

///////////////////////////////////////////////////////////////////////////////
//
// memmem()
//
// Purpose:     Find a byte sequence within a memory buffer
//
// Parameters:  buf               - pointer to buffer
//              buf_len           - size of buffer in bytes
//              byte_sequence     - byte sequence to look for
//              byte_sequence_len - size of byte sequence in bytes
//
// Returns:     void * - if successful, returns a pointer to the first 
//                       occurrence of byte_sequence in buf;  otherwise, 
//                       returns NULL
//
// Notes;       Characters in byte_sequence and characters in buf will be
//              compared "as is", with no case conversion.
//
void *memmem(const void *buf, 
			 size_t buf_len, 
			 const void *byte_sequence, 
			 size_t byte_sequence_len)
{
	BYTE *bf = (BYTE *)buf;
	BYTE *bs = (BYTE *)byte_sequence;
	BYTE *p  = bf;

	while (byte_sequence_len <= (buf_len - (p - bf)))
	{
		UINT b = *bs & 0xFF;
		if ((p = (BYTE *) memchr(p, b, buf_len - (p - bf))) != NULL)
		{
			if ((memcmp(p, byte_sequence, byte_sequence_len)) == 0)
				return p;
			else  
				p++;
		}
		else  
		{
			break;
		}
	}
	return NULL;
}