/*
* DebugDirectory.h - Part of the PeLib library.
*
* Copyright (c) 2004 Sebastian Porst (webmaster@the-interweb.com)
* All rights reserved.
*
* This software is licensed under the zlib/libpng License.
* For more details see http://www.opensource.org/licenses/zlib-license.php
* or the license information file (license.htm) in the root directory 
* of PeLib.
*/

#ifndef DEBUGDIRECTORY_H
#define DEBUGDIRECTORY_H

namespace PeLib
{
	/// Class that handles the Debug directory.
	class DebugDirectory
	{
		private:
		  /// Stores the various DebugDirectory structures.
		  std::vector<PELIB_IMG_DEBUG_DIRECTORY> m_vDebugInfo;

		public:
		  /// Reads the Debug directory from a file.
		  int read(const std::string& strFilename, unsigned int uiOffset, unsigned int uiSize);
		  /// Rebuilds the current Debug directory.
		  void rebuild(std::vector<byte>& obBuffer) const;
		  /// Returns the size the current Debug directory needs after rebuilding.
		  unsigned int size() const;
		  /// Writes the current Debug directory back to a file.
		  int write(const std::string& strFilename, unsigned int uiOffset) const;

		  /// Returns the number of DebugDirectory image structures in the current DebugDirectory.
		  unsigned int getNumberOfEntries() const;
		  
		  /// Adds a new debug structure.
		  void addEntry();
		  /// Removes a debug structure.
		  void removeEntry(unsigned int uiIndex);
		  
		  /// Returns the Characteristics value of a debug structure.
		  dword getCharacteristics(unsigned int uiIndex) const;
		  /// Returns the TimeDateStamp value of a debug structure.
		  dword getTimeDateStamp(unsigned int uiIndex) const;
		  /// Returns the MajorVersion value of a debug structure.
		  word getMajorVersion(unsigned int uiIndex) const;
		  /// Returns the MinorVersion value of a debug structure.
		  word getMinorVersion(unsigned int uiIndex) const;
		  /// Returns the Type value of a debug structure.
		  dword getType(unsigned int uiIndex) const;
		  /// Returns the SizeOfData value of a debug structure.
		  dword getSizeOfData(unsigned int uiIndex) const;
		  /// Returns the AddressOfRawData value of a debug structure.
		  dword getAddressOfRawData(unsigned int uiIndex) const;
		  /// Returns the PointerToRawData value of a debug structure.
		  dword getPointerToRawData(unsigned int uiIndex) const;
		  
		  /// Sets the Characteristics value of a debug structure.
		  void setCharacteristics(unsigned int uiIndex, dword dwValue);
		  /// Sets the TimeDateStamp value of a debug structure.
		  void setTimeDateStamp(unsigned int uiIndex, dword dwValue);
		  /// Sets the MajorVersion value of a debug structure.
		  void setMajorVersion(unsigned int uiIndex, word wValue);
		  /// Sets the MinorVersion value of a debug structure.
		  void setMinorVersion(unsigned int uiIndex, word wValue);
		  /// Sets the Type value of a debug structure.
		  void setType(unsigned int uiIndex, dword dwValue);
		  /// Sets the SizeOfData value of a debug structure.
		  void setSizeOfData(unsigned int uiIndex, dword dwValue);
		  /// Sets the AddressOfRawData value of a debug structure.
		  void setAddressOfRawData(unsigned int uiIndex, dword dwValue);
		  /// Sets the PointerToRawData value of a debug structure.
		  void setPointerToRawData(unsigned int uiIndex, dword dwValue);
	};
}
#endif
