/*
* ExportDirectory.h - Part of the PeLib library.
*
* Copyright (c) 2004 Sebastian Porst (webmaster@the-interweb.com)
* All rights reserved.
*
* This software is licensed under the zlib/libpng License.
* For more details see http://www.opensource.org/licenses/zlib-license.php
* or the license information file (license.htm) in the root directory 
* of PeLib.
*/

#ifndef EXPORTDIRECTORY_H
#define EXPORTDIRECTORY_H
#include "PeHeader.h"

namespace PeLib
{
	/// Class that handles the export directory.
	/**
	* This class handles the export directory.
	* \todo getNameString
	**/
//	template<int bits>
	class ExportDirectory
	{
		private:
		  /// Used to store all necessary information about a file's exported functions.
		  PELIB_IMAGE_EXP_DIRECTORY m_ied;

		public:
		  /// Add another function to be exported.
		  void addFunction(const std::string& strFuncname, dword dwFuncAddr);
		  /// Identifies a function through it's name.
		  int getFunctionIndex(const std::string& strFunctionName) const;
		  /// Read a file's export directory.
		  int read(const std::string& strFilename, unsigned int uiOffset, unsigned int uiSize, const PeHeader& pehHeader);
		  /// Rebuild the current export directory.
		  void rebuild(std::vector<byte>& vBuffer, dword dwRva) const;
		  /// Returns the size of the current export directory.
		  unsigned int size() const;
		  /// Writes the current export directory to a file.
		  int write(const std::string& strFilename, unsigned int uiOffset, unsigned int uiRva) const;

		  /// Changes the name of the file (according to the export directory).
		  void setNameString(const std::string& strFilename);
		  std::string getNameString() const;

		  /// Get the name of an exported function.
		  std::string getFunctionName(dword dwIndex) const;
		  /// Get the ordinal of an exported function.
		  word getFunctionOrdinal(dword dwIndex) const;
		  /// Get the address of the name of an exported function.
		  dword getAddressOfName(dword dwIndex) const;
		  /// Get the address of an exported function.
		  dword getAddressOfFunction(dword dwIndex) const;
		  
		  /// Change the name of an exported function.
		  void setFunctionName(dword dwIndex, const std::string& strName);
		  /// Change the ordinal of an exported function.
		  void setFunctionOrdinal(dword dwIndex, word wValue);
		  /// Change the address of the name of an exported function.
		  void setAddressOfName(dword dwIndex, dword dwValue);
		  /// Change the address of an exported function.
		  void setAddressOfFunction(dword dwIndex, dword dwValue);
		  
		  /*
		  word getFunctionOrdinal(std::string strFuncname) const;
		  dword getAddressOfName(std::string strFuncname) const;
		  dword getAddressOfFunction(std::string strFuncname) const;

		  void setFunctionOrdinal(std::string strFuncname, word wValue);
		  void setAddressOfName(std::string strFuncname, dword dwValue);
		  void setAddressOfFunction(std::string strFuncname, dword dwValue);
		  */

		  /// Return the Base value of the export directory.
		  dword getBase() const;
		  /// Return the Characteristics value of the export directory.
		  dword getCharacteristics() const;
		  /// Return the TimeDateStamp value of the export directory.
		  dword getTimeDateStamp() const;
		  /// Return the MajorVersion value of the export directory.
		  word getMajorVersion() const;
		  /// Return the MinorVersion value of the export directory.
		  word getMinorVersion() const;
		  /// Return the Name value of the export directory.
		  dword getName() const;
		  /// Return the NumberOfFunctions value of the export directory.
		  dword getNumberOfFunctions() const;
		  /// Return the NumberOfNames value of the export directory.
		  dword getNumberOfNames() const;
		  /// Return the AddressOfFunctions value of the export directory.
		  dword getAddressOfFunctions() const;
		  /// Return the AddressOfNames value of the export directory.
		  dword getAddressOfNames() const;

		  /// Returns the number of NameOrdinals.
		  dword getNumberOfNameOrdinals() const;
		  /// Returns the number of AddressOfFunctionNames values.
		  dword getNumberOfAddressOfFunctionNames() const;
		  /// Returns the number of AddressOfFunction values.
		  dword getNumberOfAddressOfFunctions() const;
		  /// Returns the AddressOfNameOrdinals value.
		  dword getAddressOfNameOrdinals() const;

		  /// Set the Base value of the export directory.
		  void setBase(dword dwValue);
		  /// Set the Characteristics value of the export directory.
		  void setCharacteristics(dword dwValue);
		  /// Set the TimeDateStamp value of the export directory.
		  void setTimeDateStamp(dword dwValue);
		  /// Set the MajorVersion value of the export directory.
		  void setMajorVersion(word wValue);
		  /// Set the MinorVersion value of the export directory.
		  void setMinorVersion(word wValue);
		  /// Set the Name value of the export directory.
		  void setName(dword dwValue);
		  /// Set the NumberOfFunctions value of the export directory.
		  void setNumberOfFunctions(dword dwValue);
		  /// Set the NumberOfNames value of the export directory.
		  void setNumberOfNames(dword dwValue);
		  /// Set the AddressOfFunctions value of the export directory.
		  void setAddressOfFunctions(dword dwValue);
		  /// Set the AddressOfNames value of the export directory.
		  void setAddressOfNames(dword dwValue);
	};
}
#endif
