/*
* TlsDirectory.cpp - Part of the PeLib library.
*
* Copyright (c) 2004 Sebastian Porst (webmaster@the-interweb.com)
* All rights reserved.
*
* This software is licensed under the zlib/libpng License.
* For more details see http://www.opensource.org/licenses/zlib-license.php
* or the license information file (license.htm) in the root directory 
* of PeLib.
*/

#ifdef false

#include "PeLibInc.h"
#include "TlsDirectory.h"

namespace PeLib
{
	/**
	* Reads a file's TLS directory.
	* @param strFilename Name of the file.
	* @param uiOffset File offset of the TLS directory.
	* @param uiSize Size of the TLS directory.
	**/
	int TlsDirectory::read(const std::string& strFilename, unsigned int uiOffset, unsigned int uiSize)
	{
		if (uiSize < PELIB_IMAGE_TLS_DIRECTORY::size())
		{
//			throw Exceptions::InvalidFormat(TlsDirectoryId, __LINE__);
			return 1;
		}

		std::ifstream ifFile(strFilename.c_str(), std::ios::binary);
		unsigned int ulFileSize = fileSize(ifFile);

		if (!ifFile)
		{
//			throw Exceptions::CannotOpenFile(TlsDirectoryId, __LINE__);
			return 1;
		}
		
		if (ulFileSize < uiOffset + uiSize)
		{
//			throw Exceptions::InvalidFormat(TlsDirectoryId, __LINE__);
			return 1;
		}

		ifFile.seekg(uiOffset, std::ios::beg);

		std::vector<byte> vTlsDirectory(uiSize);
		ifFile.read(reinterpret_cast<char*>(&vTlsDirectory[0]), uiSize);
		
		InputBuffer ibBuffer(vTlsDirectory);
		
		PELIB_IMAGE_TLS_DIRECTORY itdCurr;
		
		ibBuffer >> itdCurr.StartAddressOfRawData;
		ibBuffer >> itdCurr.EndAddressOfRawData;
		ibBuffer >> itdCurr.AddressOfIndex;
		ibBuffer >> itdCurr.AddressOfCallBacks;
		ibBuffer >> itdCurr.SizeOfZeroFill;
		ibBuffer >> itdCurr.Characteristics;
		
		std::swap(itdCurr, m_tls);
		
		return 0;
	}
	
	/**
	* Rebuilds the current TLS Directory.
	* @param vBuffer Buffer where the TLS directory will be written to.
	**/
	void TlsDirectory::rebuild(std::vector<byte>& vBuffer) const
	{
		OutputBuffer obBuffer(vBuffer);
		
		obBuffer << m_tls.StartAddressOfRawData;
		obBuffer << m_tls.EndAddressOfRawData;
		obBuffer << m_tls.AddressOfIndex;
		obBuffer << m_tls.AddressOfCallBacks;
		obBuffer << m_tls.SizeOfZeroFill;
		obBuffer << m_tls.Characteristics;
	}

	/**
	* Returns the size of the TLS directory. Due to the static nature of this structure the return value
	* will always be 24.
	* @return Size in bytes.
	**/
	unsigned int TlsDirectory::size() const
	{
		return PELIB_IMAGE_TLS_DIRECTORY::size();
	}

	/**
	* @param strFilename Name of the file.
	* @param dwOffset File offset the TLS Directory will be written to.
	**/
	void TlsDirectory::write(const std::string& strFilename, unsigned int dwOffset) const throw (Exceptions::CannotOpenFile, std::exception)
	{
		std::fstream ofFile(strFilename.c_str(), std::ios::in | std::ios::out | std::ios::binary);

		if (!ofFile)
		{
			ofFile.clear();
			ofFile.open(strFilename.c_str(), std::ios_base::binary | std::ios_base::in | std::ios_base::out | std::ios_base::trunc);
			if (!ofFile)
			{
				throw Exceptions::CannotOpenFile(TlsDirectoryId, __LINE__);
			}
		}

		ofFile.seekp(dwOffset, std::ios::beg);

		std::vector<unsigned char> vBuffer;
		rebuild(vBuffer);

		ofFile.write(reinterpret_cast<const char*>(&vBuffer[0]), vBuffer.size());

		ofFile.close();
	}

	/**
	* @return The StartAddressOfRawData value of the TLS directory.
	**/
	dword TlsDirectory::getStartAddressOfRawData() const
	{
		return m_tls.StartAddressOfRawData;
	}
	
	/**
	* @return The EndAddressOfRawData value of the TLS directory.
	**/
	dword TlsDirectory::getEndAddressOfRawData() const
	{
		return m_tls.EndAddressOfRawData;
	}
	
	/**
	* @return The AddressOfIndex value of the TLS directory.
	**/
	dword TlsDirectory::getAddressOfIndex() const
	{
		return m_tls.AddressOfIndex;
	}
	
	/**
	* @return The AddressOfCallBacks value of the TLS directory.
	**/
	dword TlsDirectory::getAddressOfCallBacks() const
	{
		return m_tls.AddressOfCallBacks;
	}
	
	/**
	* @return The SizeOfZeroFill value of the TLS directory.
	**/
	dword TlsDirectory::getSizeOfZeroFill() const
	{
		return m_tls.SizeOfZeroFill;
	}
	
	/**
	* @return The Characteristics value of the TLS directory.
	**/
	dword TlsDirectory::getCharacteristics() const
	{
		return m_tls.Characteristics;
	}

	/**
	* @param dwValue The new StartAddressOfRawData value of the TLS directory.
	**/
	void TlsDirectory::setStartAddressOfRawData(dword dwValue)
	{
		m_tls.StartAddressOfRawData = dwValue;
	}
	
	/**
	* @param dwValue The new EndAddressOfRawData value of the TLS directory.
	**/
	void TlsDirectory::setEndAddressOfRawData(dword dwValue)
	{
		m_tls.EndAddressOfRawData = dwValue;
	}
	
	/**
	* @param dwValue The new AddressOfIndex value of the TLS directory.
	**/
	void TlsDirectory::setAddressOfIndex(dword dwValue)
	{
		m_tls.AddressOfIndex = dwValue;
	}
	
	/**
	* @param dwValue The new AddressOfCallBacks value of the TLS directory.
	**/
	void TlsDirectory::setAddressOfCallBacks(dword dwValue)
	{
		m_tls.AddressOfCallBacks = dwValue;
	}
	
	/**
	* @param dwValue The new SizeOfZeroFill value of the TLS directory.
	**/
	void TlsDirectory::setSizeOfZeroFill(dword dwValue)
	{
		m_tls.SizeOfZeroFill = dwValue;
	}
	
	/**
	* @param dwValue The new Characteristics value of the TLS directory.
	**/
	void TlsDirectory::setCharacteristics(dword dwValue)
	{
		m_tls.Characteristics = dwValue;
	}
	

}

#endif
